from sqlalchemy import create_engine #Python2を推奨
import sqlite3
import pandas as pd
from sqlite3 import Error
 
 
 
def sqlite_connect(db_file):
    """ create a database connection to a SQLite database """
    conn = sqlite3.connect(db_file)
    return conn

def sqlite_read(query, conn):
    """ Read from a SQLite database """
    cur = conn.cursor()
    cur.execute(query) 
    result = cur.fetchall()
    return result

def mysql_connect(db, user_id=None, pw=None):
    '''Connect to the given database '''
    conn_string = ""
    if user_id != None:
        conn_string = "mysql+pymysql://" + user_id + ":" + pw + "@localhost/" + db
    else:
        conn_string = "mysql+pymysql://localhost/" + db
    
    engine = create_engine(conn_string)
    db_connection = engine.connect()
    return db_connection

def read_df(sqlStatement, table, db_connection):
    '''SQL select statement '''
    df = pd.read_sql(sqlStatement, con=db_connection)
    db_connection.close()
    return df

def insert_df(df,table, db_connection):
    '''Insert dataframe in the table '''
    msg = df.to_sql(name=table,con=db_connection,if_exists='append') 

def read(sqlStatement, table, db_connection):
    '''SQL select statement '''
    item = get(sqlStatement, con=db_connection)
    db_connection.close()
    return item

def table_exists(table, conn):
    ''' Checks that a table exists. Returns a Boolean.'''
    check = "SELECT name FROM sqlite_master WHERE type='table' AND name='%d';" % table
    result = sqlite_read(check, conn)
    return len(result) > 0

def delete(conn, table, column, value):
    query = "DELETE FROM '{0}' WHERE {1} LIKE '{2}%'".format(table, column, value)
    cur = conn.cursor()
    cur.execute(query)
    conn.commit()

def drop_table(conn, table):
    query = "DROP TABLE '{0}'".format(table)
    cur = conn.cursor()
    cur.execute(query)
    conn.commit()



import numpy as np

def negatives(data, column, regex=False):
    '''
    DEPRECATED - REPLACED by filter + formula
    Returns total count of negative values 
    '''
    bool_mask = ""
    if regex :
        bool_mask = data.filter(regex=(column))<0
    else :
        bool_mask = data.filter(column)<0
    fails = bool_mask.replace(False,np.NaN)
    
    return fails

def column(df, column, regex=False):
    '''
    Returns dataframe with only the selected column(s) 
    '''
    if regex :
        df = df.filter(regex=(column))
    else :
        df = df[column]
    
    return df



import pandas as pd
from modules.customers_info import mierukagamenxls as kanrishito
from modules.database import database


def get_timeslot_mean(customer, start, end, pulse_rate):
    '''Get yearly mean for the given timeslot'''
    weekly_cons = get_weekly_cons(customer, pulse_rate)
    timeslot = weekly_cons[start:end]
    return timeslot.cons.mean()

###################################################
bullshit_loader = kanrishito.get_cgroup_infos("ネガワット事業", ['ビルID', 'ユーザー名', 'パルスレート'])
timeslot_yearly_mean(bullshit_loader)

def timeslot_yearly_mean(dataframe):
    '''Returns the average of the weekday DR happened on '''

    # setup the df
    ids = data.index.values.tolist()
    results = pd.DataFrame(columns=["name","2019-04-10", "2019-09-09", "2019-09-10"], index=ids)


    # Calculate mean for each customer, for each day
    for idx in customers_info:
        
        print(customer, end= "      \r")
        results.loc[customer.ビルID, "name"] = customer.ユーザー名
        
        for dr_day in dr:
            if database.table_exists(customer.ビルID, conn):
                avg = get_timeslot_mean(customer.ビルID, dr_day["start"], dr_day["end"], customer.パルスレート)
                results.loc[customer.ビルID, dr_day["day"]] = avg
    return results



conn = database.sqlite_connect('./negawatt.db')

# Get DR dates and start/end times
dr = []
dr.append(dict({"day" : "2019-04-10", "start" : "Wednesday 16:00", "end": "Wednesday 19:00"}))
dr.append(dict({"day" : "2019-09-09", "start" : "Monday 16:00", "end": "Monday 19:00"}))
dr.append(dict({"day" : "2019-09-10", "start" : "Tuesday 16:00", "end": "Tuesday 19:00"}))

results = get_dr_WoD_avg()
# Clean the DF from non encodable chars
results.name = results.name.str.replace("\u3231", "") # （株）creates a bug
results.to_csv("year_avgs.csv", encoding="shift-jis") # Change to cp932 ?




from .filters import filter_to
    
def failures_count(data):
    '''
    Parameters: data, the dataframe to process
    Returns: occurences of negative values
    Returns:
        1. [Filter] for negative values (returns boolean mask)
        2. [Count] the occurences of negative values
     Note:
        失敗：調整力 <= 0  
    '''
    #.pipe: has to expect series, df or groupby as objects
    
    countNonNull = lambda df : df.count(axis="columns")
    
    (data
    .pipe(filter_to.negatives, arg1="調整電力量*", arg2=True)
    .pipe(countNonNull, arg1="columns")
    )    
    
    #fails = (data, column=, regex=True)    
    #failures_count = fails.count(axis="columns")

    return data

data = pd.DataFrame()
failures(data)



from .filters import filter_to
from .formulas import sumup

# Filters : allow to select (selectors)
contract = lambda df: filter_to.column(df, column="契約調整電力")
reduction = lambda df: filter_to.columns(df, column="^調整電力量*", regex=True)

# Formula SPECIFIC TO DEMAND RESPONSE
# A formula can be a metric (in this case, no pipeline)
def success(contract, adjust):
    '''
    Calculates the success rate of the customers.
    '''
    accomp = adjust/contract    
    return accomp


# GENERAL OPERATION
def average(df):
    # Type: Operation
    return df.groupby(df.index).mean()


import pandas as pd
data= pd.DataFrame()
AverageSuccess = data.pipe(success).pipe(average)




from filters import DayOfWeek, TimeOfDay
from formulas import Average


customers_info = kanrishito.get_cgroup_infos("ネガワット事業", ['ビルID', 'ユーザー名', 'パルスレート'])

filter_params = dict(
    {
        "filters": [
            {
                "DayOfWeek": "2019-04-10",
                "Timeslot": "16:00-18:00"
            },
            {
                "filters": ["DayOfWeek", "Timeslot"],
                "value": ["2019-09-09", "16:00-18:00"]
            },
            {
                "filters": ["DayOfWeek", "Timeslot"],
                "value": ["2019-09-10", "16:00-18:00"]
            }
        ]
})
TimeslotYearlyAverage(rawdata_df, filter_params)

def TimeslotYearlyAverage(rawdata_df, filter_params):

    """
    formules:
    Transform/Aggregation : groupby: day of week
    WeekOfDay average over a day
    
    Timeslot filter

    """




# ==============================

import pandas as pd
from modules.customers_info import mierukagamenxls as kanrishito
from modules.database import database

def get_timeslot_mean(customer, start, end, pulse_rate):
    '''Get yearly mean for the given timeslot'''
    weekly_cons = get_weekly_cons(customer, pulse_rate)
    timeslot = weekly_cons[start:end]
    return timeslot.cons.mean()


customers_info = kanrishito.get_cgroup_infos("ネガワット事業", ['ビルID', 'ユーザー名', 'パルスレート'])
timeslot_yearly_mean(customers_info)


def timeslot_yearly_mean(customers_info):
    '''Returns the average of the weekday DR happened on '''

    # Create blank dataframe
    ids = customers_info.index.values.to_list()
    base_df = pd.DataFrame(columns=["name","2019-04-10", "2019-09-09", "2019-09-10"], index=ids)

    # Calculate mean for each customer, for each day
    for customer in customers_info:
        
        print(customer, end= "      \r")
        base_df.loc[customer.ビルID, "name"] = customer.ユーザー名
        
        for dr_day in dr:
            if database.table_exists(customer.ビルID, conn):
                avg = get_timeslot_mean(customer.ビルID, dr_day["start"], dr_day["end"], customer.パルスレート)
                base_df.loc[customer.ビルID, dr_day["day"]] = avg
    return base_df


# ================ DATA ==============
conn = database.sqlite_connect('./negawatt.db')

# Get DR dates and start/end times
dr = []
dr.append(dict({"day" : "2019-04-10", "start" : "Wednesday 16:00", "end": "Wednesday 19:00"}))
dr.append(dict({"day" : "2019-09-09", "start" : "Monday 16:00", "end": "Monday 19:00"}))
dr.append(dict({"day" : "2019-09-10", "start" : "Tuesday 16:00", "end": "Tuesday 19:00"}))
# =====================================


results = get_dr_WoD_avg()
# Clean the DF from non encodable chars
results.name = results.name.str.replace("\u3231", "") # （株）creates a bug
results.to_csv("year_avgs.csv", encoding="shift-jis") # Change to cp932 ?




import pandas as pd
from .modules import database 

def from_3sheets_excel(fpath="./2019結果.xlsx"):
    '''Read from the excel summary file '''
    results = pd.read_excel(fpath, sheet_name=events, index_col=1, dtype={"電気契約番号": "string"})    
    return results

def from_sqlite(table, fpath="../../database/negawatt_infos.db"):
    conn = database.sqlite_connect(fpath)
    results = database.sqlite_read("SELECT * FROM '%s'" % table, conn)
    return results

def from_pandas(table, fpath="../../database/negawatt_infos.db"):
    conn = database.sqlite_connect(fpath)
    results = pd.read_sql("SELECT * FROM '%s'" % table, conn)
    results.index = results[results.columns[0]]
    return results



from .filters import filter_to

def Total_Adjustment(df):
    ''' Returns total adjusted power'''
    adjust = filter_to.column(df, column="^調整電力量", regex=True)
    total_per_id = adjust.groupby(df.index).sum()
    total = total_per_id.sum(axis="columns")

    print("Total Adjustment \n\n", total, "\n\n\n\n")
    return total


import yaml
from compass.core.report_frame import Report

def read_yaml(filepath):
    ''' Reads yaml to return a report'''

    txt = open(filepath, "r"). read()
    specs = yaml.load(txt, Loader=yaml.CLoader)

    rp = Report(specs)

    return rp




import pandas as pd

class Metric():
    
    def __init__(self, filters=dict(), formulas=dict()):
        self.filters = filters
        self.formulas = formulas
    
    def generate(self):
        for formula in self.formulas :
            pass




class Report():

    """
    Collection of metrics organized into a Pandas dataframe.

    Parameters
    ----------
    data : ndarray (structured or homogeneous), Iterable, dict, or DataFrame
        Dict can contain Series, arrays, constants, or list-like objects.
        .. versionchanged:: 0.23.0
           If data is a dict, column order follows insertion-order for
           Python 3.6 and later.

    See Also
    --------
    read_table : Read general delimited file into DataFrame.
    read_csv : Read a comma-separated values (csv) file into DataFrame.
    DataFrame.from_dict : From dicts of Series, arrays, or dicts.
        
    Examples
    --------
    Constructing DataFrame from a dictionary.
    >>> d = {'col1': [1, 2], 'col2': [3, 4]}
    return
    """

    def __init__(self, metrics=[], datasource=dict(), data=pd.DataFrame()):
        self.metrics = metrics
        self.datasource = datasource
        self.data = data
    
    def generate(self):
        for metric in self.metrics :
            pass
    


import pandas as pd

def get_weekly_cons(customer, pulse_rate):    
    '''Returns the average of the weekday DR happened on '''
    data = pd.read_sql("SELECT DISTINCT * FROM '%s'" % str(customer), conn, index_col="DateTime", parse_dates=True)
    data.index = pd.to_datetime(data.index)
    data["cons"] = data["PulseCount"].diff().dropna().shift(-1) *pulse_rate
    weekly_cons = data.groupby(df.index.strftime('%A %H:%M')).mean()
    return weekly_cons